/**
 ** Miwin's Dice implementation.
 **/

pif_DiceSimulator/MiwinDice
	parent_type = /pif_DiceSimulator/Dice

	/*
	 * A set of non-transitive dice. There are actually multiple sets of Miwin's Dice, but I'm only
     setting up the first.
	 */

	III
		// III has an advantage against IV, and a disadvantage against V.

		New()
			return 1

		Maximum()
			return 9
		Minimum()
			return 1

		Size()
			return 6
		Sides()
			return list(1, 2, 5, 6, 7, 9)

		Roll()
			switch(rand(1,6))
				if(1) return 1
				if(2) return 2
				if(3) return 5
				if(4) return 6
				if(5) return 7
				if(6) return 9

	IV
		// IV has an advantage against V and a disadvantage against III.

		New()
			return 1

		Maximum()
			return 9
		Minimum()
			return 1

		Size()
			return 6
		Sides()
			return list(1, 3, 4, 5, 8, 9)

		Roll()
			switch(rand(1,6))
				if(1) return 1
				if(2) return 3
				if(3) return 4
				if(4) return 4
				if(5) return 8
				if(6) return 9

	V
		// V has an advantage against III and a disadvantage against IV.

		New()
			return 1

		Maximum()
			return 8
		Minimum()
			return 2

		Size()
			return 6
		Sides()
			return list(2, 3, 4, 6, 7, 8)

		Roll()
			switch(rand(1,6))
				if(1) return 2
				if(2) return 3
				if(3) return 4
				if(4) return 6
				if(5) return 7
				if(6) return 8

mob/verb
	MiwinRoll()
		set category = "Miwin's Dice"

		var/pif_DiceSimulator/MiwinDice
			III/III = new
			IV/IV = new
			V/V = new

		// Almost heresy, but it's just a demo.
		III = III.Roll()
		IV = IV.Roll()
		V = V.Roll()

		world << "<tt>Rolls:<tt>"
		world << "<tt>\tIII:\t[III]</tt>"
		world << "<tt>\tIV:\t[IV]</tt>"
		world << "<tt>\tV:\t[V]</tt>"
		world << ""
		world << "<tt>\tResults:</tt>"

		if(III > IV)
			world << "<tt>\t\t- III beats IV.</tt>"
		else if(III == IV)
			world << "<tt>\t\t- III ties IV.</tt>"
		else
			world << "<tt>\t\t- III loses to IV.</tt>"

		if(IV > V)
			world << "<tt>\t\t- IV beats V.</tt>"
		else if(IV == V)
			world << "<tt>\t\t- IV ties V.</tt>"
		else
			world << "<tt>\t\t- IV loses to V.</tt>"

		if(III < V)
			world << "<tt>\t\t- III loses to V.</tt>"
		else if(III == V)
			world << "<tt>\t\t- III tied V.</tt>"
		else
			world << "<tt>\t\t- III beats V.</tt>"

		world << ""

	MiwinRoll_Info()
		set category = "Miwin's Dice"

		world << "<tt>Miwin's dice are a set of non-transitive dice, labelled III, IV, and V. III is 'good against' IV, IV is 'good against' " + \
				 "V and V is 'good against' III, each which a probablity of 17/36.</tt>\n"

/**
 ** Coin flips, with various levels of bias for heads.
 **/

pif_DiceSimulator/Coin
	unbiased
		parent_type = /pif_DiceSimulator/coin

	slightly_biased
		// Heads is more likely than tails, with an 11/20 chance.

		parent_type = /pif_DiceSimulator/coin

		Roll()
			if(rand(1,20) > 9)
				return 1
			else
				return 0

	biased
		// Heads is more likely than tails, with a 3/5 chance.

		parent_type = /pif_DiceSimulator/coin

		Roll()
			if(rand(1,5) > 2)
				return 1
			else
				return 0

	very_biased
		// Heads is more likely than tails, with a 2/3 chance.

		parent_type = /pif_DiceSimulator/coin

		Roll()
			if(rand(1,3) > 1)
				return 1
			else
				return 0

	absurdly_biased
		// Heads is more likely than tails, with a 99/100 chance.

		parent_type = /pif_DiceSimulator/coin

		Roll()
			if(rand(1,100) > 1)
				return 1
			else
				return 0

mob/verb
	UnbiasedCoinFlip()
		set category = "Coins"

		var/pif_DiceSimulator/Coin/unbiased/c = new
		var/S = "Unbiased Coin (10 times): "
		for(var/i in 1 to 10)
			S += "[c.Map()], "
		world << "<tt>[copytext(S, 1, length(S)-1)]</tt>"

	SlightlyBiasedCoinFlip()
		set category = "Coins"

		var/pif_DiceSimulator/Coin/slightly_biased/c = new
		var/S = "Slightly Biased Coin (10 times): "
		for(var/i in 1 to 10)
			S += "[c.Map()], "
		world << "<tt>[copytext(S, 1, length(S)-1)]</tt>"

	BiasedCoinFlip()
		set category = "Coins"

		var/pif_DiceSimulator/Coin/biased/c = new
		var/S = "Biased Coin (10 times): "
		for(var/i in 1 to 10)
			S += "[c.Map()], "
		world << "<tt>[copytext(S, 1, length(S)-1)]</tt>"

	VeryBiasedCoinFlip()
		set category = "Coins"

		var/pif_DiceSimulator/Coin/very_biased/c = new
		var/S = "Very Biased Coin (10 times): "
		for(var/i in 1 to 10)
			S += "[c.Map()], "
		world << "<tt>[copytext(S, 1, length(S)-1)]</tt>"

	AbsurdlyBiasedCoinFlip()
		set category = "Coins"

		var/pif_DiceSimulator/Coin/absurdly_biased/c = new
		var/S = "Absurdly Biased Coin (10 times): "
		for(var/i in 1 to 10)
			S += "[c.Map()], "
		world << "<tt>[copytext(S, 1, length(S)-2)]</tt>"

	CoinFlip_Explanation()
		set category = "Coins"

		world << "<tt>These verbs simulate flipping a coin with various biases towards the heads side. These are:</tt>"
		world << "<tt>\t- Unbiased: 1/2 chance</tt>"
		world << "<tt>\t- Slightly Biased: 11/20 chance</tt>"
		world << "<tt>\t- Biased: 3/5 chance</tt>"
		world << "<tt>\t- Very Biased: 2/3 chance</tt>"
		world << "<tt>\t- Absurdly Biased: 99/100 chance</tt>"

/**
 ** Ways of drawing 2 die from 4d6, with a 2d6 as a comparison. Ord(i,j) will denote summing the
 ** lowest i, j die. E.g., Ord(1,2) denotes summing the lowest and second-lowest die.
 **/

mob/verb

	/*
	 * Regualar 2d6, as a "control"
	 */

	Roll_2d6()
		set name = "Roll 2d6"
		set category = "d6 Roll"

		var/pif_DiceSimulator/Collection/c = new("2d6")
		var/S = "2d6 for ten rolls: "
		for(var/i in 1 to 10)
			c.Roll()
			S += "[c.Sum()], "
		world << "<tt>[copytext(S, 1, length(S)-1)]</tt>"

	Frequency_2d6()
		set name = "2d6"
		set category = "d6 Frequency"

		var
			pif_DiceSimulator/Collection/c = new("2d6")

			list/frequencies = list()
		for(var/i in 1 to 12)
			frequencies.len ++
			frequencies[i] = 0

		for(var/i in 1 to 10000)
			c.Roll()
			frequencies[c.Sum()]++

		var/S = "Simulated frequencies for 2d6:\n"
		for(var/i in 2 to 12)
			S += "[i]\t"
			var/amount = round(frequencies[i]/50)
			for(var/j in 1 to amount)
				S += "="
			S += "\n"

		world << "<tt>[S]\n</tt>"

	/*
	 * 4d6 Ord(1,2)
	 */

	Roll_4d6Ord12()
		set category = "d6 Roll"
		set name="Roll 4d6 Ord(1,2)"

		var/pif_DiceSimulator/Collection/c = new("4d6")
		var/S = "4d6 Ord(1,2) for ten rolls: "
		for(var/i in 1 to 10)
			c.Roll()
			S += "[c.At(1) + c.At(2)], "
		world << "<tt>[copytext(S, 1, length(S)-1)]</tt>"

	Frequency_4d6Ord12()
		set category = "d6 Frequency"
		set name = "4d6 Ord(1,2)"

		var
			pif_DiceSimulator/Collection/c = new("4d6")

			list/frequencies = list()
		for(var/i in 1 to 12)
			frequencies.len ++
			frequencies[i] = 0

		for(var/i in 1 to 10000)
			c.Roll()
			frequencies[c.At(1) + c.At(2)]++

		var/S = "Simulated frequencies for 4d6 Ord(1,2):\n"
		for(var/i in 2 to 12)
			S += "[i]\t"
			var/amount = round(frequencies[i]/50)
			for(var/j in 1 to amount)
				S += "="
			S += "\n"

		world << "<tt>[S]\n</tt>"

	/*
	 * 4d6 Ord(1,3)
	 */

	Roll_4d6Ord13()
		set category = "d6 Roll"
		set name="Roll 4d6 Ord(1,3)"

		var/pif_DiceSimulator/Collection/c = new("4d6")
		var/S = "4d6 Ord(1,3) for ten rolls: "
		for(var/i in 1 to 10)
			c.Roll()
			S += "[c.At(1) + c.At(3)], "
		world << "<tt>[copytext(S, 1, length(S)-1)]</tt>"

	Frequency_4d6Ord13()
		set category = "d6 Frequency"
		set name = "4d6 Ord(1,3)"

		var
			pif_DiceSimulator/Collection/c = new("4d6")

			list/frequencies = list()
		for(var/i in 1 to 12)
			frequencies.len ++
			frequencies[i] = 0

		for(var/i in 1 to 10000)
			c.Roll()
			frequencies[c.At(1) + c.At(3)]++

		var/S = "Simulated frequencies for 4d6 Ord(1,3):\n"
		for(var/i in 2 to 12)
			S += "[i]\t"
			var/amount = round(frequencies[i]/50)
			for(var/j in 1 to amount)
				S += "="
			S += "\n"

		world << "<tt>[S]\n</tt>"

	/*
	 * 4d6 Ord(1,4)
	 */

	Roll_4d6Ord14()
		set category = "d6 Roll"
		set name="Roll 4d6 Ord(1,4)"

		var/pif_DiceSimulator/Collection/c = new("4d6")
		var/S = "4d6 Ord(1,4) for ten rolls: "
		for(var/i in 1 to 10)
			c.Roll()
			S += "[c.At(1) + c.At(4)], "
		world << "<tt>[copytext(S, 1, length(S)-1)]</tt>"

	Frequency_4d6Ord14()
		set category = "d6 Frequency"
		set name = "4d6 Ord(1,4)"

		var
			pif_DiceSimulator/Collection/c = new("4d6")

			list/frequencies = list()
		for(var/i in 1 to 12)
			frequencies.len ++
			frequencies[i] = 0

		for(var/i in 1 to 10000)
			c.Roll()
			frequencies[c.At(1) + c.At(4)]++

		var/S = "Simulated frequencies for 4d6 Ord(1,4):\n"
		for(var/i in 2 to 12)
			S += "[i]\t"
			var/amount = round(frequencies[i]/50)
			for(var/j in 1 to amount)
				S += "="
			S += "\n"

		world << "<tt>[S]\n</tt>"

	/*
	 * 4d6 Ord(2,3)
	 */

	Roll_4d6Ord23()
		set category = "d6 Roll"
		set name="Roll 4d6 Ord(2,3)"

		var/pif_DiceSimulator/Collection/c = new("4d6")
		var/S = "4d6 Ord(2,3) for ten rolls: "
		for(var/i in 1 to 10)
			c.Roll()
			S += "[c.At(2) + c.At(3)], "
		world << "<tt>[copytext(S, 1, length(S)-1)]</tt>"

	Frequency_4d6Ord23()
		set category = "d6 Frequency"
		set name = "4d6 Ord(2,3)"

		var
			pif_DiceSimulator/Collection/c = new("4d6")

			list/frequencies = list()
		for(var/i in 1 to 12)
			frequencies.len ++
			frequencies[i] = 0

		for(var/i in 1 to 10000)
			c.Roll()
			frequencies[c.At(2) + c.At(3)]++

		var/S = "Simulated frequencies for 4d6 Ord(2,3):\n"
		for(var/i in 2 to 12)
			S += "[i]\t"
			var/amount = round(frequencies[i]/50)
			for(var/j in 1 to amount)
				S += "="
			S += "\n"

		world << "<tt>[S]\n</tt>"

	/*
	 * 4d6 Ord(2,4)
	 */

	Roll_4d6Ord24()
		set category = "d6 Roll"
		set name="Roll 4d6 Ord(2,4)"

		var/pif_DiceSimulator/Collection/c = new("4d6")
		var/S = "4d6 Ord(2,4) for ten rolls: "
		for(var/i in 1 to 10)
			c.Roll()
			S += "[c.At(2) + c.At(4)], "
		world << "<tt>[copytext(S, 1, length(S)-1)]</tt>"

	Frequency_4d6Ord24()
		set category = "d6 Frequency"
		set name = "4d6 Ord(2,4)"

		var
			pif_DiceSimulator/Collection/c = new("4d6")

			list/frequencies = list()
		for(var/i in 1 to 12)
			frequencies.len ++
			frequencies[i] = 0

		for(var/i in 1 to 10000)
			c.Roll()
			frequencies[c.At(2) + c.At(4)]++

		var/S = "Simulated frequencies for 4d6 Ord(2,4):\n"
		for(var/i in 2 to 12)
			S += "[i]\t"
			var/amount = round(frequencies[i]/50)
			for(var/j in 1 to amount)
				S += "="
			S += "\n"

		world << "<tt>[S]\n</tt>"

	/*
	 * 4d6 Ord(3,4)
	 */

	Roll_4d6Ord34()
		set category = "d6 Roll"
		set name="Roll 4d6 Ord(3,4)"

		var/pif_DiceSimulator/Collection/c = new("4d6")
		var/S = "4d6 Ord(3,4) for ten rolls: "
		for(var/i in 1 to 10)
			c.Roll()
			S += "[c.At(3) + c.At(4)], "
		world << "<tt>[copytext(S, 1, length(S)-1)]</tt>"

	Frequency_4d6Ord34()
		set category = "d6 Frequency"
		set name = "4d6 Ord(3,4)"

		var
			pif_DiceSimulator/Collection/c = new("4d6")

			list/frequencies = list()
		for(var/i in 1 to 12)
			frequencies.len ++
			frequencies[i] = 0

		for(var/i in 1 to 10000)
			c.Roll()
			frequencies[c.At(3) + c.At(4)]++

		var/S = "Simulated frequencies for 4d6 Ord(3,4):\n"
		for(var/i in 2 to 12)
			S += "[i]\t"
			var/amount = round(frequencies[i]/50)
			for(var/j in 1 to amount)
				S += "="
			S += "\n"

		world << "<tt>[S]\n</tt>"

	/*
	 * Explanation of frequency output and what the rolling means.
	 */

	Roll_Explanation()
		set category = "d6 Roll"

		world << "<tt>This simulates the rolling of various six-sided die and taking their sum. 2d6 is the 'baseline', and \
			      is simply the sum of two six-sided die. The others are drawn from a pool of four six-sided die, but instead \
			      of taking the sum immediately, they are arranged in increasing order and drawn in a certain way. For example, \
			      Ord(1,2) indicates that the smallest and second-smallest are drawn and summed, while Ord(1,4) indicates the \
			      smallest and larger are drawn and summed.</tt>"

	Frequency_Explanation()
		set category = "d6 Frequency"

		world << "<tt>This generates a simulated frequency distribution chart of various types of dice rolls (see the 'Roll \
				  Explanation' verb in 'd6 Roll' for info on what the notation means). By <i>simulated</i>, I mean that the \
				  result is computed after simulating 10,000 rolls, rather than outright determining the probabilities beforehand \
				  and showing a chart. Consequently, you'll see some discrepancies in the charts between simulations, but as \
				  one would expend the general trends are the same.</tt>"